from django.utils.module_loading import import_string
from django.contrib.auth import get_user_model
from rest_framework import generics


from .settings import api_settings


User = get_user_model()


class TokenUserDetailView(generics.RetrieveAPIView):
    serializer_class = None
    _serializer_class = api_settings.USER_MODEL_SERIALIZER

    def get_serializer_class(self):
        """
        If serializer_class is set when overridden, use it, otherwise get the class from settings.
        """

        if self.serializer_class:
            return self.serializer_class  # type: ignore

        try:
            return import_string(self._serializer_class)
        except ImportError:
            msg = f"Could not import serializer '{self._serializer_class}'"
            raise ImportError(msg)

    def get_object(self):
        return self.request.user



from rest_framework import serializers
from django.contrib.auth import get_user_model

User = get_user_model()


class TokenUserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = (
            "id",
            "first_name",
            "email",
            "last_name",
            "date_joined",
            "last_login",
            "is_active",
            "is_staff",
            "is_superuser",
        )
        read_only_fields = ("date_joined", "last_login", "is_staff", "is_superuser")



from typing import Any, Tuple
from django.conf import settings
from django.test.signals import setting_changed
from rest_framework.settings import APISettings as _APISettings


USER_SETTINGS = getattr(settings, "EASY_JWT", None)

DEFAULTS = {
    "USER_ID_FIELD": "id",
    "USER_ID_CLAIM": "user_id",
    "USER_MODEL_SERIALIZER": "easyjwt_user.serializers.TokenUserSerializer",
}

IMPORT_STRINGS = ()

REMOVED_SETTINGS: Tuple[str] = ("TEST",)

EASY_JWT: dict[str, Any] = {}


class APISettings(_APISettings):  # pragma: no cover
    def __check_user_settings(self, user_settings):
        SETTINGS_DOC = "https://django-easyjwt.readthedocs.io/en/latest/settings.html"

        for setting in REMOVED_SETTINGS:
            if setting in user_settings:
                raise RuntimeError(
                    ("The '{}' setting has been removed. Please refer to '{}' for available settings."),
                    setting,
                    SETTINGS_DOC,
                )

        return user_settings


api_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)


def reload_api_settings(*args, **kwargs):  # pragma: no cover
    global api_settings

    setting, value = kwargs["setting"], kwargs["value"]
    if setting == "EASY_JWT":
        api_settings = APISettings(value, DEFAULTS, IMPORT_STRINGS)


setting_changed.connect(reload_api_settings)



# Generated by Django 5.0.3 on 2025-01-25 07:40

import django.utils.timezone
import easyjwt_user.models
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('first_name', models.CharField(blank=True, max_length=150, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('location', models.TextField(blank=True, null=True)),
                ('phone', models.CharField(blank=True, max_length=15, null=True)),
                ('email', models.EmailField(error_messages={'unique': 'This email already exists.'}, help_text='Please Enter valid Email Address', max_length=254, verbose_name='email address')),
                ('modified_date', models.DateTimeField(auto_now=True, verbose_name='modified date')),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name='creation date')),
                ('jwt_secret', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
            ],
            options={
                'verbose_name': 'user',
                'verbose_name_plural': 'users',
                'db_table': 'user',
                'abstract': False,
                'swappable': 'AUTH_USER_MODEL',
            },
            managers=[
                ('objects', easyjwt_user.models.UserManager()),
            ],
        ),
        migrations.AddConstraint(
            model_name='user',
            constraint=models.UniqueConstraint(fields=('email',), name='user_email_type_unique'),
        ),
    ]



from rest_framework import generics, permissions, response
from django.contrib.auth.views import PasswordChangeView as BasePasswordChangeView
from django.views.generic.base import RedirectView
from django.views.generic import TemplateView
from django.urls import reverse_lazy
from django.conf import settings
from django.http import HttpResponseRedirect


from .serializers import (
    TokenObtainPairSerializer,
    TokenRefreshSerializer,
    TokenVerifySerializer,
)
from .utils import TokenManager


class PasswordChangeDoneView(TemplateView):
    template_name = "registration/password_change_done.html"


class PasswordChangeView(BasePasswordChangeView):
    template_name = "registration/password_change_form.html"  # Custom template
    success_url = reverse_lazy("password_change_done")  # Redirect after success

    def post(self, request):
        tokenmanager = TokenManager()
        data = request.POST.copy()
        username = getattr(request.user, tokenmanager.username_field)
        tokenmanager.password_change(
            username,
            data.get("old_password"),
            data.get("new_password1"),
        )
        return HttpResponseRedirect(self.get_success_url())


class PasswordResetView(RedirectView):
    # Optional: Set whether the redirect is permanent
    permanent = False  # Set to True for a permanent redirect (301)
    # Optional: Include query strings in the redirect
    query_string = True  # Set to False to ignore query strings

    # Optional: Dynamically generate the redirect URL
    def get_redirect_url(self, *args, **kwargs):
        return f"{settings.EASY_JWT['REMOTE_AUTH_SERVICE_URL']}/accounts/password_reset/"


class TokenObtainPairView(generics.CreateAPIView):
    permission_classes = (permissions.AllowAny,)
    serializer_class = TokenObtainPairSerializer

    def post(self, request):
        serializer = TokenObtainPairSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        tokenmanager = TokenManager()
        tokens = tokenmanager.authenticate(**serializer.validated_data)

        return response.Response(tokens)


class TokenRefreshView(generics.CreateAPIView):
    permission_classes = (permissions.AllowAny,)
    serializer_class = TokenRefreshSerializer

    def post(self, request):
        serializer = TokenRefreshSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        tokenmanager = TokenManager()
        tokens = tokenmanager.refresh(**serializer.validated_data)

        return response.Response(tokens)


class TokenVerifyView(generics.CreateAPIView):
    permission_classes = (permissions.AllowAny,)
    serializer_class = TokenVerifySerializer

    def post(self, request):
        serializer = TokenVerifySerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        tokenmanager = TokenManager()
        tokenmanager.verify(**serializer.validated_data)

        return response.Response({})



from django.urls import path, include
from .views import TokenUserDetailView


urlpatterns = [
    path(
        "user/",
        include(
            [
                path("", TokenUserDetailView.as_view(), name="user_detail"),
            ]
        ),
    ),
]



from typing import List

from .models import User

from django import forms
from django.contrib import admin
from django.contrib.auth.models import Group
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import ReadOnlyPasswordHashField
from django.contrib.admin.widgets import FilteredSelectMultiple


class UserCreationForm(forms.ModelForm):
    """A form for creating new users. Includes all the required
    fields, plus a repeated password."""

    password1 = forms.CharField(label="Password", widget=forms.PasswordInput)
    password2 = forms.CharField(label="Password confirmation", widget=forms.PasswordInput)

    class Meta:
        model = User
        fields = ("email",)

    def clean_password2(self):
        # Check that the two password entries match
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError("Passwords don't match")
        return password2

    def save(self, commit=True):
        # Save the provided password in hashed format
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class GroupAdminForm(forms.ModelForm):
    class Meta:
        model = Group
        exclude: List[str] = []

    # Add the users field.
    users = forms.ModelMultipleChoiceField(
        queryset=User.objects.all(),
        required=False,
        # Use the pretty 'filter_horizontal widget'.
        widget=FilteredSelectMultiple("users", False),
    )

    def __init__(self, *args, **kwargs):
        # Do the normal form initialisation.
        super(GroupAdminForm, self).__init__(*args, **kwargs)
        # If it is an existing group (saved objects have a pk).
        if self.instance.pk:
            # Populate the users field with the current Group users.
            self.fields["users"].initial = self.instance.user_set.all()

    def save_m2m(self):
        # Add the users to the Group.
        self.instance.user_set.set(self.cleaned_data["users"])

    def save(self, *args, **kwargs):
        # Default save
        instance = super(GroupAdminForm, self).save()
        # Save many-to-many data
        self.save_m2m()
        return instance


class UserChangeForm(forms.ModelForm):
    """A form for updating users. Includes all the fields on
    the user, but replaces the password field with admin's
    password hash display field.
    """

    password = ReadOnlyPasswordHashField(
        label=("Password"),
        help_text=(
            "Plain text passwords are not stored, so there is no way to see "
            "this user's password, but you can change the password "
            'using <a href="../password/">this form</a>.'
        ),
    )

    class Meta:
        model = User
        fields = ("is_active", "is_superuser", "password")

    def clean_password(self):
        # Regardless of what the user provides, return the initial value.
        # This is done here, rather than on the field, because the
        # field does not have access to the initial value
        return self.initial["password"]


@admin.register(User)
class EasyJWTUserAdmin(UserAdmin):
    # The forms to add and change user instances
    form = UserChangeForm
    add_form = UserCreationForm
    # The fields to be used in displaying the User model.
    # These override the definitions on the base UserAdmin
    # that reference specific fields on auth.User.
    list_display = ("email", "is_active", "is_superuser")
    list_filter = (
        "is_active",
        "is_superuser",
    )
    fieldsets = (
        (None, {"fields": ("email", "first_name", "last_name", "phone", "password")}),
        (
            "Permissions",
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                )
            },
        ),
        ("Important dates", {"fields": ("last_login",)}),
    )
    add_fieldsets = ((None, {"classes": ("wide",), "fields": ("email", "password1", "password2")}),)
    search_fields = ("email",)
    ordering = ("email",)
    filter_horizontal = ()


# unregister the Group model from admin.
admin.site.unregister(Group)


# Create a new Group admin.
class GroupAdmin(admin.ModelAdmin):
    # Use our custom form.
    form = GroupAdminForm
    # Filter permissions horizontal as well.
    filter_horizontal = ["permissions"]


# Register the new Group ModelAdmin.
admin.site.register(Group, GroupAdmin)



from django.urls import path
from django.contrib.auth.views import LoginView
from .views import (
    TokenObtainPairView,
    TokenRefreshView,
    TokenVerifyView,
    PasswordChangeView,
    PasswordChangeDoneView,
    PasswordResetView,
)


urlpatterns = [
    path("token/", TokenObtainPairView.as_view(), name="token_obtain_pair"),
    path("token/refresh/", TokenRefreshView.as_view(), name="token_refresh"),
    path("token/verify/", TokenVerifyView.as_view(), name="token_verify"),
    path("password-change/", PasswordChangeView.as_view(), name="password_change"),
    path("password-change/done/", PasswordChangeDoneView.as_view(), name="password_change_done"),
    path("password-reset/", PasswordResetView.as_view(), name="password_reset"),
    path("login/", LoginView.as_view(), name="login"),
]
